controllers = angular.module 'rest.controllers',  ['ngResource']

controllers.controller 'ServantListCtrl', ($scope, $modal, $resource, $http, $log) ->
    resource = $resource('api/felines/:id', {}, {'update': { method:'PUT' }})
    resource.fetchCount = () ->
        $http({method: 'GET', url: 'api/felines/count'})
            .success((data) -> $scope.totalItems = data.count)
            .error((_, status) -> setErrMsg("Get/Count ERROR: #{status}"))

    $scope.errorMessage = undefined
    clearErrMsg = -> $scope.errorMessage = ""
    setErrMsg = (m) -> $scope.errorMessage = m

    $scope.selectedRow = []

    $scope.felines = []

    $scope.totalItems = 0;
    $scope.currentPage = 1;
    $scope.itemsPerPage = 5;

#   async helps callsbacks complete
    $scope.getPagedDataAsync = ->
        setTimeout(
            -> resource.query(
                {offset: ($scope.itemsPerPage * ($scope.currentPage - 1)), max: $scope.itemsPerPage, sort: "id", order: "asc"}
                (value) ->
                    $scope.felines = value
                    resource.fetchCount()
                (httpResponse) -> setErrMsg("Query ERROR: #{httpResponse.statusText}")
            )
            10
        )

    # don't like this mixing of code and template...would be better to reference an HTML template
    cellTmpl = '''
    <div class="ngCellText ng-scope col0 colt0" ng-class="col.colIndex()">
        <span ng-cell-text="" class="ng-binding">
            <button type="button" class="btn btn-default btn-xs" ng-click="rm(row.entity)">
                <span class="glyphicon glyphicon-minus"></span>
            </button>
            <button type="button" class="btn btn-default btn-xs" ng-click="edit(row.entity)">
                <span class="glyphicon glyphicon-pencil"></span>
            </button>
        </span>
    </div>
    '''

    col0HeaderCellTemplate = '''
    <div class="ngHeaderSortColumn ngCellText {{col.headerClass}}"">
        <span ng-cell-text="" class="ng-binding">
            <div>
                <button type="button" class="btn btn-default btn-xs" ng-click="create()">
                    <span class="glyphicon glyphicon-plus"></span>
                </button>
            </div>
        </span>
    </div>
    '''

    $scope.gridOptions = {
        totalServerItems: 'totalServerItems'
        data: 'felines'
        columnDefs: [
            {field:'', displayName: '', width: '64px', sortable: false, enableCellEdit: false,
                resizable: false, cellTemplate: cellTmpl, headerCellTemplate: col0HeaderCellTemplate},
            {field: 'id', displayName: 'ID', width: "**", resizable: false},
            {field: 'name', displayName: 'Name', width: "***", resizable: false}
            {field: 'age', displayName: 'Age', width: "*", resizable: false}
            {field: 'deceased', displayName: 'Dead', width: "*", resizable: false}
            {field: 'description', displayName: 'Description', width: "**********"}
        ]
        selectedItems: $scope.selectedRow
        enableSorting: false
        multiSelect: false
        showFooter: false
    }

    $scope.getPagedDataAsync()

    $scope.$on('ngGridEventData', -> $scope.gridOptions.selectRow(0, true))

    $scope.create = ->
        clearErrMsg()
        modalInstance = $modal.open({
          templateUrl: 'modalEditCreateForm.tmpl',
          controller: ModalCreateEditCtrl,
          resolve: {
              modalMode: -> 'create'
              row: ->
                  {
                  name: ""
                  age: 0
                  deceased: false
                  description: ""
                  }
          }
        })
        modalInstance.result.then((e) ->
            e.felines = []
            resource.save e,
                          (-> $scope.getPagedDataAsync()),
                          ((httpResponse) -> setErrMsg("Save ERROR: #{httpResponse.statusText}"))
        )

    $scope.rm = (e) ->
      clearErrMsg()
      modalInstance = $modal.open({
          templateUrl: 'modalRmForm.tmpl',
          controller: ModalRmCtrl,
          resolve: { row: -> e }
      })
      modalInstance.result.then((e) ->
          e.$remove {id: e.id},
                    (-> $scope.getPagedDataAsync()),
                    ((httpResponse) -> setErrMsg("Remove ERROR: #{httpResponse.statusText}"))

      )

    $scope.edit = (e) ->
        clearErrMsg()
        modalInstance = $modal.open({
          templateUrl: 'modalEditCreateForm.tmpl',
          controller: ModalCreateEditCtrl,
          resolve: {
              modalMode: -> 'edit'
              row: -> angular.copy(e)  # allows for 'cancel'
          }
        })
        modalInstance.result.then((e) ->
            oldId = e.id
            delete e[x] for x in ['class', 'felines', 'servant', 'id']
            e.$update {id: oldId},
                    (-> $scope.getPagedDataAsync()),
                    ((httpResponse) -> setErrMsg("Edit ERROR: #{httpResponse.statusText}"))
        )

    ModalRmCtrl = ($scope, $modalInstance, row) ->
        $scope.row = row
        $scope.ok = -> $modalInstance.close(row)
        $scope.cancel = -> $modalInstance.dismiss('cancel')

    ModalCreateEditCtrl = ($scope, $modalInstance, row, modalMode) ->
        $scope.row = row
        $scope.modalMode = modalMode
        $scope.ok = -> $modalInstance.close(row)
        $scope.cancel = -> $modalInstance.dismiss('cancel')

